@@ -1,6 +1,7 @@
-/* vim: set noet ts=8 sw=8 sts=0: */
-/* Tracker Extract - extracts embedded metadata from files
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/*
  * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
+ * Copyright (C) 2008, Nokia
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -20,26 +21,37 @@
 
 #include "config.h"
 
-#include <poppler.h>
 #include <string.h>
+#include <poppler.h>
+
 #include <glib.h>
 
-#include "tracker-extract.h"
+#include "tracker-main.h"
 #include "tracker-xmp.h"
 
+#include <libtracker-common/tracker-utils.h>
+
+static void extract_pdf (const gchar *filename,
+			 GHashTable  *metadata);
+
+static TrackerExtractData data[] = {
+	{ "application/pdf", extract_pdf },
+	{ NULL, NULL }
+};
 
 static void
-tracker_extract_pdf (const gchar *filename, GHashTable *metadata)
+extract_pdf (const gchar *filename,
+	     GHashTable  *metadata)
 {
 	PopplerDocument *document;
-	gchar           *tmp;
-	gchar           *title          = NULL;
-	gchar           *author         = NULL;
-	gchar           *subject        = NULL;
-	gchar           *keywords       = NULL;
-	gchar           *metadata_xml   = NULL;
-	GTime            creation_date;
-	GError          *error          = NULL;
+	gchar		*tmp;
+	gchar		*title		= NULL;
+	gchar		*author		= NULL;
+	gchar		*subject	= NULL;
+	gchar		*keywords	= NULL;
+	gchar		*metadata_xml	= NULL;
+	GTime		 creation_date;
+	GError		*error		= NULL;
 
 	g_type_init ();
 
@@ -49,43 +61,45 @@ tracker_extract_pdf (const gchar *filename, GHashTable *metadata)
 
 	if (document == NULL || error) {
 		return;
-        }
+	}
 
 	g_object_get (document,
-                      "title", &title,
-                      "author", &author,
-                      "subject", &subject,
-                      "keywords", &keywords,
-                      "creation-date", &creation_date,
-                      NULL);
+		      "title", &title,
+		      "author", &author,
+		      "subject", &subject,
+		      "keywords", &keywords,
+		      "creation-date", &creation_date,
+		      NULL);
 
 	/* metadata property not present in older poppler versions */
 	if (g_object_class_find_property (G_OBJECT_GET_CLASS (document), "metadata")) {
 		g_object_get (document, "metadata", &metadata_xml, NULL);
-        }
+	}
 
 	if (!tracker_is_empty_string (title)) {
-		g_hash_table_insert (metadata, g_strdup ("Doc:Title"), g_strdup (title));
-        }
+		g_hash_table_insert (metadata,
+				     g_strdup ("Doc:Title"),
+				     tracker_escape_metadata (title));
+	}
 	if (!tracker_is_empty_string (author)) {
-		g_hash_table_insert (metadata, g_strdup ("Doc:Author"), g_strdup (author));
-        }
+		g_hash_table_insert (metadata,
+				     g_strdup ("Doc:Author"),
+				     tracker_escape_metadata (author));
+	}
 	if (!tracker_is_empty_string (subject)) {
-		g_hash_table_insert (metadata, g_strdup ("Doc:Subject"), g_strdup (subject));
-        }
+		g_hash_table_insert (metadata,
+				     g_strdup ("Doc:Subject"),
+				     tracker_escape_metadata (subject));
+	}
 	if (!tracker_is_empty_string (keywords)) {
-		g_hash_table_insert (metadata, g_strdup ("Doc:Keywords"), g_strdup (keywords));
-        }
+		g_hash_table_insert (metadata,
+				     g_strdup ("Doc:Keywords"),
+				     tracker_escape_metadata (keywords));
+	}
 
-
-#if 0
-	GTimeVal creation_date_val = { creation_date, 0 };
-	g_hash_table_insert (metadata, g_strdup ("Doc.Created"),
-		g_time_val_to_iso8601 (creation_date_val));
-#endif
-
-	g_hash_table_insert (metadata, g_strdup ("Doc:PageCount"),
-		g_strdup_printf ("%d", poppler_document_get_n_pages (document)));
+	g_hash_table_insert (metadata,
+			     g_strdup ("Doc:PageCount"),
+			     tracker_escape_metadata_printf ("%d", poppler_document_get_n_pages (document)));
 
 	if ( metadata_xml ) {
 		tracker_read_xmp (metadata_xml, strlen (metadata_xml), metadata);
@@ -100,15 +114,8 @@ tracker_extract_pdf (const gchar *filename, GHashTable *metadata)
 	g_object_unref (document);
 }
 
-
-TrackerExtractorData data[] = {
- 	{ "application/pdf", tracker_extract_pdf },
-	{ NULL, NULL }
-};
-
-
-TrackerExtractorData *
-tracker_get_extractor_data (void)
+TrackerExtractData *
+tracker_get_extract_data (void)
 {
 	return data;
 }
